import numpy as np

from sce_helper import sort

debug_evolution = False

def evolution(partition, population_size, partition_amount, alpha, beta, cost_function):
    
    for community in partition:
        for _ in range(beta):
            m = population_size / partition_amount
            q = int(population_size/(2*partition_amount))
            
            if debug_evolution:
                print("Community size is", m)
                print("The amount of selected individuals is", q)

            selected_individuals = []
            selected_indeces = []
            
            r = np.random.uniform(low=0, high=1, size=(q-1))
            selected_indeces = (m * r ** 2).astype(int)

            selected_individuals = community[selected_indeces]

            if debug_evolution:
                print("Selected indices", selected_indeces)
                print("Selected individuals", selected_individuals)
            
            for _ in range(alpha):
                
                selected_individuals, selected_indeces = sort(selected_individuals, cost=cost_function)

                if debug_evolution:
                    print("Selected individuals are", selected_individuals)

                # Define random weights for the random gravity center
                weights = np.random.uniform(low=0, high=1, size=(1, q-1)).T
                w = np.sum(weights)
                weights = np.hstack([weights, weights])
                
                if debug_evolution:
                    print("Generated random weights", weights)

                # Comute the random gravity center
                g = np.sum(selected_individuals * weights, axis=0) / w
                
                if debug_evolution:
                    print("Gravity center should be equal to the sum of following arrays", selected_individuals*weights, "divided by", w)
                    print("Gravity center is", g)
                
                r = 2 * g - selected_individuals[-1]

                if cost_function(r[0], r[1]) < cost_function(selected_individuals[-1][0], selected_individuals[-1][1]):
                    if debug_evolution:
                        print("Contraction step.")
                    selected_individuals[-1] = r
                else :
                    if debug_evolution:
                        print("Mutation step.")
                    c = (g + selected_individuals[-1])/2
                    if cost_function(c[0], c[1]) < cost_function(selected_individuals[-1][0], selected_individuals[-1][1]):
                        if debug_evolution:
                            print("Mutated individual was better.")
                        selected_individuals[-1] = c
                    else :
                        if debug_evolution:
                            print("Mutated individual was worse.")
                        min_values = np.min(community, axis=0)
                        max_values = np.max(community, axis=0)
                        
                        weights = np.random.uniform(low=0, high=1, size=(2))
                        z = (max_values-min_values) * weights + min_values 
                        
                        if debug_evolution:
                            print("Found min values for community", min_values)
                            print("Found max values for community", max_values)
                            print("Randomly chose value within hypercube containing the community", z)
            
            community[selected_indeces] = selected_individuals
    return partition
